@@ -15,12 +15,14 @@ class JSONField(models.TextField):
     """Simple JSON field that stores python structures as JSON strings
     on database.
     """
-    __metaclass__ = models.SubfieldBase
 
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('default', '{}')
         super(JSONField, self).__init__(*args, **kwargs)
 
+    def from_db_value(self, value, expression, connection, context):
+        return self.to_python(value)
+
     def to_python(self, value):
         """
         Convert the input JSON value into python structures, raises
@@ -33,6 +35,10 @@ class JSONField(models.TextField):
             value = six.text_type(value, 'utf-8')
         if isinstance(value, six.string_types):
             try:
+                # with django 1.6 i have '"{}"' as default value here
+                if value[0] == value[-1] == '"':
+                    value = value[1:-1]
+
                 return json.loads(value)
             except Exception as err:
                 raise ValidationError(str(err))
