@@ -8,7 +8,7 @@ import json
 import base64
 import hashlib
 
-from social.utils import parse_qs, constant_time_compare
+from social.utils import parse_qs, constant_time_compare, handle_http_errors
 from social.backends.oauth import BaseOAuth2
 from social.exceptions import AuthException, AuthCanceled, AuthUnknownError, \
                               AuthMissingParameter
@@ -19,11 +19,11 @@ class FacebookOAuth2(BaseOAuth2):
     name = 'facebook'
     RESPONSE_TYPE = None
     SCOPE_SEPARATOR = ','
-    AUTHORIZATION_URL = 'https://www.facebook.com/dialog/oauth'
-    ACCESS_TOKEN_URL = 'https://graph.facebook.com/oauth/access_token'
-    REVOKE_TOKEN_URL = 'https://graph.facebook.com/{uid}/permissions'
+    AUTHORIZATION_URL = 'https://www.facebook.com/v2.3/dialog/oauth'
+    ACCESS_TOKEN_URL = 'https://graph.facebook.com/v2.3/oauth/access_token'
+    REVOKE_TOKEN_URL = 'https://graph.facebook.com/v2.3/{uid}/permissions'
     REVOKE_TOKEN_METHOD = 'DELETE'
-    USER_DATA_URL = 'https://graph.facebook.com/me'
+    USER_DATA_URL = 'https://graph.facebook.com/v2.3/me'
     EXTRA_DATA = [
         ('id', 'id'),
         ('expires', 'expires')
@@ -46,6 +46,14 @@ class FacebookOAuth2(BaseOAuth2):
         """Loads user data from service"""
         params = self.setting('PROFILE_EXTRA_PARAMS', {})
         params['access_token'] = access_token
+
+        if self.setting('APPSECRET_PROOF', True):
+            _, secret = self.get_key_and_secret()
+            params['appsecret_proof'] = hmac.new(
+                secret.encode('utf8'),
+                msg=access_token.encode('utf8'),
+                digestmod=hashlib.sha256
+            ).hexdigest()
         return self.get_json(self.USER_DATA_URL, params=params)
 
     def process_error(self, data):
@@ -54,6 +62,7 @@ class FacebookOAuth2(BaseOAuth2):
             raise AuthCanceled(self, data.get('error_message') or
                                      data.get('error_code'))
 
+    @handle_http_errors
     def auth_complete(self, *args, **kwargs):
         """Completes loging process, must return user instance"""
         self.process_error(self.data)
@@ -61,13 +70,19 @@ class FacebookOAuth2(BaseOAuth2):
             raise AuthMissingParameter(self, 'code')
         state = self.validate_state()
         key, secret = self.get_key_and_secret()
-        url = self.ACCESS_TOKEN_URL
-        response = self.get_querystring(url, params={
+        response = self.request(self.ACCESS_TOKEN_URL, params={
             'client_id': key,
             'redirect_uri': self.get_redirect_uri(state),
             'client_secret': secret,
             'code': self.data['code']
         })
+        # API v2.3 returns a JSON, according to the documents linked at issue
+        # #592, but it seems that this needs to be enabled(?), otherwise the
+        # usual querystring type response is returned.
+        try:
+            response = response.json()
+        except ValueError:
+            response = parse_qs(response.text)
         access_token = response['access_token']
         return self.do_auth(access_token, response, *args, **kwargs)
 
@@ -129,7 +144,7 @@ class FacebookAppOAuth2(FacebookOAuth2):
         if 'signed_request' in self.data:
             key, secret = self.get_key_and_secret()
             response = self.load_signed_request(self.data['signed_request'])
-            if not 'user_id' in response and not 'oauth_token' in response:
+            if 'user_id' not in response and 'oauth_token' not in response:
                 raise AuthException(self)
 
             if response is not None:
@@ -162,7 +177,7 @@ class FacebookAppOAuth2(FacebookOAuth2):
     def load_signed_request(self, signed_request):
         def base64_url_decode(data):
             data = data.encode('ascii')
-            data += '=' * (4 - (len(data) % 4))
+            data += '='.encode('ascii') * (4 - (len(data) % 4))
             return base64.urlsafe_b64decode(data)
 
         key, secret = self.get_key_and_secret()
@@ -172,8 +187,10 @@ class FacebookAppOAuth2(FacebookOAuth2):
             pass  # ignore if can't split on dot
         else:
             sig = base64_url_decode(sig)
-            data = json.loads(base64_url_decode(payload))
-            expected_sig = hmac.new(secret, msg=payload,
+            payload_json_bytes = base64_url_decode(payload)
+            data = json.loads(payload_json_bytes.decode('utf-8', 'replace'))
+            expected_sig = hmac.new(secret.encode('ascii'),
+                                    msg=payload.encode('ascii'),
                                     digestmod=hashlib.sha256).digest()
             # allow the signed_request to function for upto 1 day
             if constant_time_compare(sig, expected_sig) and \
